package passbaseplugin.pettersonapps.com.passbase_flutter

import android.content.Context
import android.view.View
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import com.passbase.passbase_sdk.PassbaseSDK
import com.passbase.passbase_sdk.PassbaseButton
import com.passbase.passbase_sdk.PassbaseSDKListener
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory

class PassbaseFlutterPlugin(passbase: PassbaseSDK, channel: MethodChannel): MethodCallHandler {
  private val passbaseRef: PassbaseSDK = passbase
  private val channel: MethodChannel = channel

  companion object {
    @JvmStatic
    fun registerWith(registrar: Registrar) {
      val channel = MethodChannel(registrar.messenger(), "passbase_flutter")
      val passbase =  PassbaseSDK(registrar.activity())
      channel.setMethodCallHandler(PassbaseFlutterPlugin(passbase, channel))
      PassbaseButtonPlugin.registerWith(registrar, passbase, channel)
    }
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      "initialize" -> {
        PassbaseSDK.source = 3
        passbaseRef.initialize(call.arguments as String)

        result.success(null)
      }
      "startVerification" -> {
        passbaseRef.callback(object :  PassbaseSDKListener {
          override fun onStart() {
            channel.invokeMethod("verification_start", "Verification started")
          }

          override fun onFinish(identityAccessKey: String?) {
            channel.invokeMethod("verification_complete", identityAccessKey)
          }

          override fun onError(errorCode: String) {
            channel.invokeMethod("verification_error", errorCode)
          }
        })

        passbaseRef.startVerification()
      }
      "setPrefillUserEmail" -> {
        passbaseRef.prefillUserEmail = call.arguments as String;
      }
      else -> {
        result.notImplemented()
      }
    }
  }
}

object PassbaseButtonPlugin {
  fun registerWith(registrar: Registrar, passbase: PassbaseSDK, channel: MethodChannel) {
    val viewFactory = PassbaseButtonFactory(registrar.messenger(), passbase, channel)
    registrar.platformViewRegistry()
            .registerViewFactory("PassbaseButton", viewFactory)
  }
}

class PassbaseButtonFactory(private val messenger: BinaryMessenger, passbase: PassbaseSDK, channel: MethodChannel) :
        PlatformViewFactory(StandardMessageCodec.INSTANCE) {
  private val passbase: PassbaseSDK
  private val channel: MethodChannel

  init {
    this.passbase = passbase
    this.channel = channel
  }

  override fun create(context: Context,
                      id: Int,
                      args: Any?): PlatformView {
    return PassbaseButtonContainer(context,
            passbase,
            channel,
            id,
            args)
  }
}

class PassbaseButtonContainer internal constructor(private val context: Context, passbase: PassbaseSDK, channel: MethodChannel, id: Int, args: Any?) : PlatformView {
  private val passbaseButton: PassbaseButton
  private val passbaseRef: PassbaseSDK
  private val channel: MethodChannel

  init {
    this.channel = channel
    passbaseRef = passbase
    passbaseButton = PassbaseButton(context = context)
    passbaseButton.setOnClickListener {
      passbaseRef.callback(object :  PassbaseSDKListener {
        override fun onStart() {
          channel.invokeMethod("verification_start", "Verification started")
        }

        override fun onFinish(identityAccessKey: String?) {
          channel.invokeMethod("verification_complete", identityAccessKey)
        }

        override fun onError(errorCode: String) {
          channel.invokeMethod("verification_error", errorCode)
        }
      })
      passbaseRef.startVerification()
    }
  }

  override fun dispose() {
  }

  override fun getView(): View {
    return passbaseButton
  }
}
